/*
 * case11.cpp
 *
 *  Created on: Feb 7, 2024
 *      Author: Dario
 */

/**
 * Tests if a thread with IGNORED status is properly ignored.
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_11

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>
#include <sta/tacos/watchdog.hpp>
#include <sta/rtos/signal.hpp>

namespace sta
{
    namespace tasty
    {
    	class ThreadA : public tacos::TacosThread
		{
		public:
			ThreadA()
				: tacos::TacosThread{"A", osPriorityNormal}
			{

			}

			void init() override
			{
				// Tells the watchdog to ignore this thread.
				watchdogIgnore();
			}

			void func() override
			{
				// Thread is stuck in infinite loop
				while (true)
				{
					osThreadYield();
				}
			}
		};

    	class Observer : public tacos::TacosThread
		{
		public:
			Observer()
				: tacos::TacosThread{"Observer", osPriorityNormal}
			{

			}


			void func() override
			{
				this->sleep(2000);

				// The watchdog shouldn't have restarted thread A.
				STA_TASTY_ASSERT(tacos::Watchdog::instance()->getNumRestarts() == 0);

				STA_TASTY_TERMINATE();
			}
		};

        void onTastyInit()
        {
            tacos::addThread<ThreadA>({0});
            tacos::addThread<Observer>({0, 1});
        }
    } // namespace tasty
}

#endif // TASTY_CASE_11
