/**
 * Check if the lockout timer works correctly.
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_3

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>

namespace sta
{
	namespace tasty
	{
		class DummyThread : public tacos::TacosThread
		{
		private:
			/* data */
		public:
			DummyThread()
				: tacos::TacosThread{"Dummy", osPriorityNormal}
			{}

			void func() override 
			{
				STA_TASTY_INIT();

				STA_TASTY_ASSERT(tacos::getState() == 0);

                // Trigger a state transition with lockout of 100 ticks.
                tacos::requestState(0, 1, 100);

                // The state transition should have been executed correctly.
                STA_TASTY_ASSERT(tacos::getState() == 1);

                // This state transition should get blocked.
                tacos::requestState(1, 3);
                STA_TASTY_ASSERT(tacos::getState() == 1);

                this->sleep(50);

                // This state transition should get blocked.
                tacos::requestState(1, 4);
                STA_TASTY_ASSERT(tacos::getState() == 1);

                this->sleep(44);

                // This state transition should get blocked.
                tacos::requestState(1, 2);
                STA_TASTY_ASSERT(tacos::getState() == 1);

                this->sleep(10);

                // This state transition should get executed correctly.
                tacos::requestState(1, 7);
                STA_TASTY_ASSERT(tacos::getState() == 7);

                STA_TASTY_TERMINATE();
			}
		};

		void onTastyInit()
		{
			tacos::addThread<DummyThread>({0, 1, 2, 3, 4, 5, 6, 7});
		}
	} // namespace tasty
} // namespace sta

#endif // TASTY_CASE_3
