/**
 * Tests if killing tasks works correctly and the memory is correctly freed.
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_6

#include <memory>
#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>

#include <sta/rtos/debug/heap_stats.hpp>
#include <sta/rtos/signal.hpp>

namespace sta
{
	namespace tasty
	{
		static uint8_t victim_alive_flag;


		class Victim : public tacos::TacosThread
		{
		public:
			Victim()
				: tacos::TacosThread{"Dummy", osPriorityNormal}
			{}

			void func() override
			{
				victim_alive_flag = 1;
				osThreadYield();
			}
		};


		class ViciousMurderer : public tacos::TacosThread
		{
		public:
			ViciousMurderer(std::shared_ptr<Victim> victim_ptr)
				: tacos::TacosThread{"Dummy", osPriorityNormal},
				  victim_ptr_{victim_ptr}
			{}

			void func() override
			{
				// Kill the thread and set it's heartbeat flag to 0.
				victim_ptr_->kill();
				victim_alive_flag = 0;

				uint8_t frees = sta::rtos::getNumFrees();

				// Skip a tick
				osThreadYield();

				// The killed threat shouldn't have set the heartbeat flag.
				STA_TASTY_ASSERT(victim_alive_flag == 0);

				// Wait for 100 ticks.
				this->sleep(100);

				// After waiting 100 ticks, the flag should still be 0 because the thread doesn't run.
				STA_TASTY_ASSERT(victim_alive_flag == 0);

				// Also, there should be two more frees:
				STA_TASTY_ASSERT(sta::rtos::getNumFrees() - frees == 2);

				STA_TASTY_TERMINATE();
			}
		private:
			std::shared_ptr<Victim> victim_ptr_;
		};

		void onTastyInit()
		{
			STA_TASTY_INIT();

			std::shared_ptr<Victim> victim_ptr = tacos::addThread<Victim>({0});
			tacos::addThread<ViciousMurderer>({0}, victim_ptr);
		}
	} // namespace tasty
} // namespace sta

#endif // TASTY_CASE_6/*




