/*
 * case7.cpp
 *
 *  Created on: Jan 24, 2024
 *      Author: Dario
 */

/**
 * A thread gets stuck in an infinite loop and the watchdog restarts it. 
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_7

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>
#include <sta/tacos/watchdog.hpp>

namespace sta
{
    namespace tasty
    {
        static uint8_t alive_flag = 0;

        class Dummy : public tacos::TacosThread
        {
        public:
            Dummy()
                : tacos::TacosThread{"Dummy", osPriorityNormal}
            {

            }

            void init() override
            {
                alive_flag = 1;
            }

            void func() override
            {
                this->sleep(100);

                // Thread is stuck in infinite loop
                while (true)
                {
                    osThreadYield();
                }
            }
        };

        class Observer : public tacos::TacosThread
        {
        public:
            Observer()
                : tacos::TacosThread{"Observer", osPriorityNormal}
            {

            }

            void func() override
            {
                this->sleep(10);
                STA_TASTY_ASSERT(alive_flag == 1);

                this->sleep(110);
                alive_flag = 0;

                // After at most 1000 ticks, the watchdog should have revived the thread.
                this->sleep(2000);
                STA_TASTY_ASSERT(alive_flag == 1);
                STA_TASTY_ASSERT(tacos::Watchdog::instance()->getNumRestarts() == 1);

                STA_TASTY_TERMINATE();
            }
        };

        void onTastyInit()
        {
        	STA_TASTY_INIT();

            tacos::addThread<Dummy>({0});
            tacos::addThread<Observer>({0});
        }
    } // namespace tasty
}

#endif // TASTY_CASE_7


